#ifdef SQLITE_VERSION_NUMBER
{"SQLITE_VERSION_NUMBER", SQLITE_VERSION_NUMBER},
#endif
#ifdef SQLITE_ABORT
{"SQLITE_ABORT", SQLITE_ABORT},
#endif
#ifdef SQLITE_AUTH
{"SQLITE_AUTH", SQLITE_AUTH},
#endif
#ifdef SQLITE_BUSY
{"SQLITE_BUSY", SQLITE_BUSY},
#endif
#ifdef SQLITE_CANTOPEN
{"SQLITE_CANTOPEN", SQLITE_CANTOPEN},
#endif
#ifdef SQLITE_CONSTRAINT
{"SQLITE_CONSTRAINT", SQLITE_CONSTRAINT},
#endif
#ifdef SQLITE_CORRUPT
{"SQLITE_CORRUPT", SQLITE_CORRUPT},
#endif
#ifdef SQLITE_DONE
{"SQLITE_DONE", SQLITE_DONE},
#endif
#ifdef SQLITE_EMPTY
{"SQLITE_EMPTY", SQLITE_EMPTY},
#endif
#ifdef SQLITE_ERROR
{"SQLITE_ERROR", SQLITE_ERROR},
#endif
#ifdef SQLITE_FORMAT
{"SQLITE_FORMAT", SQLITE_FORMAT},
#endif
#ifdef SQLITE_FULL
{"SQLITE_FULL", SQLITE_FULL},
#endif
#ifdef SQLITE_INTERNAL
{"SQLITE_INTERNAL", SQLITE_INTERNAL},
#endif
#ifdef SQLITE_INTERRUPT
{"SQLITE_INTERRUPT", SQLITE_INTERRUPT},
#endif
#ifdef SQLITE_IOERR
{"SQLITE_IOERR", SQLITE_IOERR},
#endif
#ifdef SQLITE_LOCKED
{"SQLITE_LOCKED", SQLITE_LOCKED},
#endif
#ifdef SQLITE_MISMATCH
{"SQLITE_MISMATCH", SQLITE_MISMATCH},
#endif
#ifdef SQLITE_MISUSE
{"SQLITE_MISUSE", SQLITE_MISUSE},
#endif
#ifdef SQLITE_NOLFS
{"SQLITE_NOLFS", SQLITE_NOLFS},
#endif
#ifdef SQLITE_NOMEM
{"SQLITE_NOMEM", SQLITE_NOMEM},
#endif
#ifdef SQLITE_NOTADB
{"SQLITE_NOTADB", SQLITE_NOTADB},
#endif
#ifdef SQLITE_NOTFOUND
{"SQLITE_NOTFOUND", SQLITE_NOTFOUND},
#endif
#ifdef SQLITE_NOTICE
{"SQLITE_NOTICE", SQLITE_NOTICE},
#endif
#ifdef SQLITE_OK
{"SQLITE_OK", SQLITE_OK},
#endif
#ifdef SQLITE_PERM
{"SQLITE_PERM", SQLITE_PERM},
#endif
#ifdef SQLITE_PROTOCOL
{"SQLITE_PROTOCOL", SQLITE_PROTOCOL},
#endif
#ifdef SQLITE_RANGE
{"SQLITE_RANGE", SQLITE_RANGE},
#endif
#ifdef SQLITE_READONLY
{"SQLITE_READONLY", SQLITE_READONLY},
#endif
#ifdef SQLITE_ROW
{"SQLITE_ROW", SQLITE_ROW},
#endif
#ifdef SQLITE_SCHEMA
{"SQLITE_SCHEMA", SQLITE_SCHEMA},
#endif
#ifdef SQLITE_TOOBIG
{"SQLITE_TOOBIG", SQLITE_TOOBIG},
#endif
#ifdef SQLITE_WARNING
{"SQLITE_WARNING", SQLITE_WARNING},
#endif
#ifdef SQLITE_ABORT_ROLLBACK
{"SQLITE_ABORT_ROLLBACK", SQLITE_ABORT_ROLLBACK},
#endif
#ifdef SQLITE_AUTH_USER
{"SQLITE_AUTH_USER", SQLITE_AUTH_USER},
#endif
#ifdef SQLITE_BUSY_RECOVERY
{"SQLITE_BUSY_RECOVERY", SQLITE_BUSY_RECOVERY},
#endif
#ifdef SQLITE_BUSY_SNAPSHOT
{"SQLITE_BUSY_SNAPSHOT", SQLITE_BUSY_SNAPSHOT},
#endif
#ifdef SQLITE_CANTOPEN_CONVPATH
{"SQLITE_CANTOPEN_CONVPATH", SQLITE_CANTOPEN_CONVPATH},
#endif
#ifdef SQLITE_CANTOPEN_FULLPATH
{"SQLITE_CANTOPEN_FULLPATH", SQLITE_CANTOPEN_FULLPATH},
#endif
#ifdef SQLITE_CANTOPEN_ISDIR
{"SQLITE_CANTOPEN_ISDIR", SQLITE_CANTOPEN_ISDIR},
#endif
#ifdef SQLITE_CANTOPEN_NOTEMPDIR
{"SQLITE_CANTOPEN_NOTEMPDIR", SQLITE_CANTOPEN_NOTEMPDIR},
#endif
#ifdef SQLITE_CONSTRAINT_CHECK
{"SQLITE_CONSTRAINT_CHECK", SQLITE_CONSTRAINT_CHECK},
#endif
#ifdef SQLITE_CONSTRAINT_COMMITHOOK
{"SQLITE_CONSTRAINT_COMMITHOOK", SQLITE_CONSTRAINT_COMMITHOOK},
#endif
#ifdef SQLITE_CONSTRAINT_FOREIGNKEY
{"SQLITE_CONSTRAINT_FOREIGNKEY", SQLITE_CONSTRAINT_FOREIGNKEY},
#endif
#ifdef SQLITE_CONSTRAINT_FUNCTION
{"SQLITE_CONSTRAINT_FUNCTION", SQLITE_CONSTRAINT_FUNCTION},
#endif
#ifdef SQLITE_CONSTRAINT_NOTNULL
{"SQLITE_CONSTRAINT_NOTNULL", SQLITE_CONSTRAINT_NOTNULL},
#endif
#ifdef SQLITE_CONSTRAINT_PRIMARYKEY
{"SQLITE_CONSTRAINT_PRIMARYKEY", SQLITE_CONSTRAINT_PRIMARYKEY},
#endif
#ifdef SQLITE_CONSTRAINT_ROWID
{"SQLITE_CONSTRAINT_ROWID", SQLITE_CONSTRAINT_ROWID},
#endif
#ifdef SQLITE_CONSTRAINT_TRIGGER
{"SQLITE_CONSTRAINT_TRIGGER", SQLITE_CONSTRAINT_TRIGGER},
#endif
#ifdef SQLITE_CONSTRAINT_UNIQUE
{"SQLITE_CONSTRAINT_UNIQUE", SQLITE_CONSTRAINT_UNIQUE},
#endif
#ifdef SQLITE_CONSTRAINT_VTAB
{"SQLITE_CONSTRAINT_VTAB", SQLITE_CONSTRAINT_VTAB},
#endif
#ifdef SQLITE_CORRUPT_VTAB
{"SQLITE_CORRUPT_VTAB", SQLITE_CORRUPT_VTAB},
#endif
#ifdef SQLITE_IOERR_ACCESS
{"SQLITE_IOERR_ACCESS", SQLITE_IOERR_ACCESS},
#endif
#ifdef SQLITE_IOERR_AUTH
{"SQLITE_IOERR_AUTH", SQLITE_IOERR_AUTH},
#endif
#ifdef SQLITE_IOERR_BLOCKED
{"SQLITE_IOERR_BLOCKED", SQLITE_IOERR_BLOCKED},
#endif
#ifdef SQLITE_IOERR_CHECKRESERVEDLOCK
{"SQLITE_IOERR_CHECKRESERVEDLOCK", SQLITE_IOERR_CHECKRESERVEDLOCK},
#endif
#ifdef SQLITE_IOERR_CLOSE
{"SQLITE_IOERR_CLOSE", SQLITE_IOERR_CLOSE},
#endif
#ifdef SQLITE_IOERR_CONVPATH
{"SQLITE_IOERR_CONVPATH", SQLITE_IOERR_CONVPATH},
#endif
#ifdef SQLITE_IOERR_DELETE
{"SQLITE_IOERR_DELETE", SQLITE_IOERR_DELETE},
#endif
#ifdef SQLITE_IOERR_DELETE_NOENT
{"SQLITE_IOERR_DELETE_NOENT", SQLITE_IOERR_DELETE_NOENT},
#endif
#ifdef SQLITE_IOERR_DIR_CLOSE
{"SQLITE_IOERR_DIR_CLOSE", SQLITE_IOERR_DIR_CLOSE},
#endif
#ifdef SQLITE_IOERR_DIR_FSYNC
{"SQLITE_IOERR_DIR_FSYNC", SQLITE_IOERR_DIR_FSYNC},
#endif
#ifdef SQLITE_IOERR_FSTAT
{"SQLITE_IOERR_FSTAT", SQLITE_IOERR_FSTAT},
#endif
#ifdef SQLITE_IOERR_FSYNC
{"SQLITE_IOERR_FSYNC", SQLITE_IOERR_FSYNC},
#endif
#ifdef SQLITE_IOERR_GETTEMPPATH
{"SQLITE_IOERR_GETTEMPPATH", SQLITE_IOERR_GETTEMPPATH},
#endif
#ifdef SQLITE_IOERR_LOCK
{"SQLITE_IOERR_LOCK", SQLITE_IOERR_LOCK},
#endif
#ifdef SQLITE_IOERR_MMAP
{"SQLITE_IOERR_MMAP", SQLITE_IOERR_MMAP},
#endif
#ifdef SQLITE_IOERR_NOMEM
{"SQLITE_IOERR_NOMEM", SQLITE_IOERR_NOMEM},
#endif
#ifdef SQLITE_IOERR_RDLOCK
{"SQLITE_IOERR_RDLOCK", SQLITE_IOERR_RDLOCK},
#endif
#ifdef SQLITE_IOERR_READ
{"SQLITE_IOERR_READ", SQLITE_IOERR_READ},
#endif
#ifdef SQLITE_IOERR_SEEK
{"SQLITE_IOERR_SEEK", SQLITE_IOERR_SEEK},
#endif
#ifdef SQLITE_IOERR_SHMLOCK
{"SQLITE_IOERR_SHMLOCK", SQLITE_IOERR_SHMLOCK},
#endif
#ifdef SQLITE_IOERR_SHMMAP
{"SQLITE_IOERR_SHMMAP", SQLITE_IOERR_SHMMAP},
#endif
#ifdef SQLITE_IOERR_SHMOPEN
{"SQLITE_IOERR_SHMOPEN", SQLITE_IOERR_SHMOPEN},
#endif
#ifdef SQLITE_IOERR_SHMSIZE
{"SQLITE_IOERR_SHMSIZE", SQLITE_IOERR_SHMSIZE},
#endif
#ifdef SQLITE_IOERR_SHORT_READ
{"SQLITE_IOERR_SHORT_READ", SQLITE_IOERR_SHORT_READ},
#endif
#ifdef SQLITE_IOERR_TRUNCATE
{"SQLITE_IOERR_TRUNCATE", SQLITE_IOERR_TRUNCATE},
#endif
#ifdef SQLITE_IOERR_UNLOCK
{"SQLITE_IOERR_UNLOCK", SQLITE_IOERR_UNLOCK},
#endif
#ifdef SQLITE_IOERR_VNODE
{"SQLITE_IOERR_VNODE", SQLITE_IOERR_VNODE},
#endif
#ifdef SQLITE_IOERR_WRITE
{"SQLITE_IOERR_WRITE", SQLITE_IOERR_WRITE},
#endif
#ifdef SQLITE_LOCKED_SHAREDCACHE
{"SQLITE_LOCKED_SHAREDCACHE", SQLITE_LOCKED_SHAREDCACHE},
#endif
#ifdef SQLITE_NOTICE_RECOVER_ROLLBACK
{"SQLITE_NOTICE_RECOVER_ROLLBACK", SQLITE_NOTICE_RECOVER_ROLLBACK},
#endif
#ifdef SQLITE_NOTICE_RECOVER_WAL
{"SQLITE_NOTICE_RECOVER_WAL", SQLITE_NOTICE_RECOVER_WAL},
#endif
#ifdef SQLITE_READONLY_CANTLOCK
{"SQLITE_READONLY_CANTLOCK", SQLITE_READONLY_CANTLOCK},
#endif
#ifdef SQLITE_READONLY_DBMOVED
{"SQLITE_READONLY_DBMOVED", SQLITE_READONLY_DBMOVED},
#endif
#ifdef SQLITE_READONLY_RECOVERY
{"SQLITE_READONLY_RECOVERY", SQLITE_READONLY_RECOVERY},
#endif
#ifdef SQLITE_READONLY_ROLLBACK
{"SQLITE_READONLY_ROLLBACK", SQLITE_READONLY_ROLLBACK},
#endif
#ifdef SQLITE_WARNING_AUTOINDEX
{"SQLITE_WARNING_AUTOINDEX", SQLITE_WARNING_AUTOINDEX},
#endif
#ifdef SQLITE_OPEN_AUTOPROXY
{"SQLITE_OPEN_AUTOPROXY", SQLITE_OPEN_AUTOPROXY},
#endif
#ifdef SQLITE_OPEN_CREATE
{"SQLITE_OPEN_CREATE", SQLITE_OPEN_CREATE},
#endif
#ifdef SQLITE_OPEN_DELETEONCLOSE
{"SQLITE_OPEN_DELETEONCLOSE", SQLITE_OPEN_DELETEONCLOSE},
#endif
#ifdef SQLITE_OPEN_EXCLUSIVE
{"SQLITE_OPEN_EXCLUSIVE", SQLITE_OPEN_EXCLUSIVE},
#endif
#ifdef SQLITE_OPEN_FULLMUTEX
{"SQLITE_OPEN_FULLMUTEX", SQLITE_OPEN_FULLMUTEX},
#endif
#ifdef SQLITE_OPEN_MAIN_DB
{"SQLITE_OPEN_MAIN_DB", SQLITE_OPEN_MAIN_DB},
#endif
#ifdef SQLITE_OPEN_MAIN_JOURNAL
{"SQLITE_OPEN_MAIN_JOURNAL", SQLITE_OPEN_MAIN_JOURNAL},
#endif
#ifdef SQLITE_OPEN_MASTER_JOURNAL
{"SQLITE_OPEN_MASTER_JOURNAL", SQLITE_OPEN_MASTER_JOURNAL},
#endif
#ifdef SQLITE_OPEN_MEMORY
{"SQLITE_OPEN_MEMORY", SQLITE_OPEN_MEMORY},
#endif
#ifdef SQLITE_OPEN_NOMUTEX
{"SQLITE_OPEN_NOMUTEX", SQLITE_OPEN_NOMUTEX},
#endif
#ifdef SQLITE_OPEN_PRIVATECACHE
{"SQLITE_OPEN_PRIVATECACHE", SQLITE_OPEN_PRIVATECACHE},
#endif
#ifdef SQLITE_OPEN_READONLY
{"SQLITE_OPEN_READONLY", SQLITE_OPEN_READONLY},
#endif
#ifdef SQLITE_OPEN_READWRITE
{"SQLITE_OPEN_READWRITE", SQLITE_OPEN_READWRITE},
#endif
#ifdef SQLITE_OPEN_SHAREDCACHE
{"SQLITE_OPEN_SHAREDCACHE", SQLITE_OPEN_SHAREDCACHE},
#endif
#ifdef SQLITE_OPEN_SUBJOURNAL
{"SQLITE_OPEN_SUBJOURNAL", SQLITE_OPEN_SUBJOURNAL},
#endif
#ifdef SQLITE_OPEN_TEMP_DB
{"SQLITE_OPEN_TEMP_DB", SQLITE_OPEN_TEMP_DB},
#endif
#ifdef SQLITE_OPEN_TEMP_JOURNAL
{"SQLITE_OPEN_TEMP_JOURNAL", SQLITE_OPEN_TEMP_JOURNAL},
#endif
#ifdef SQLITE_OPEN_TRANSIENT_DB
{"SQLITE_OPEN_TRANSIENT_DB", SQLITE_OPEN_TRANSIENT_DB},
#endif
#ifdef SQLITE_OPEN_URI
{"SQLITE_OPEN_URI", SQLITE_OPEN_URI},
#endif
#ifdef SQLITE_OPEN_WAL
{"SQLITE_OPEN_WAL", SQLITE_OPEN_WAL},
#endif
#ifdef SQLITE_IOCAP_ATOMIC
{"SQLITE_IOCAP_ATOMIC", SQLITE_IOCAP_ATOMIC},
#endif
#ifdef SQLITE_IOCAP_ATOMIC16K
{"SQLITE_IOCAP_ATOMIC16K", SQLITE_IOCAP_ATOMIC16K},
#endif
#ifdef SQLITE_IOCAP_ATOMIC1K
{"SQLITE_IOCAP_ATOMIC1K", SQLITE_IOCAP_ATOMIC1K},
#endif
#ifdef SQLITE_IOCAP_ATOMIC2K
{"SQLITE_IOCAP_ATOMIC2K", SQLITE_IOCAP_ATOMIC2K},
#endif
#ifdef SQLITE_IOCAP_ATOMIC32K
{"SQLITE_IOCAP_ATOMIC32K", SQLITE_IOCAP_ATOMIC32K},
#endif
#ifdef SQLITE_IOCAP_ATOMIC4K
{"SQLITE_IOCAP_ATOMIC4K", SQLITE_IOCAP_ATOMIC4K},
#endif
#ifdef SQLITE_IOCAP_ATOMIC512
{"SQLITE_IOCAP_ATOMIC512", SQLITE_IOCAP_ATOMIC512},
#endif
#ifdef SQLITE_IOCAP_ATOMIC64K
{"SQLITE_IOCAP_ATOMIC64K", SQLITE_IOCAP_ATOMIC64K},
#endif
#ifdef SQLITE_IOCAP_ATOMIC8K
{"SQLITE_IOCAP_ATOMIC8K", SQLITE_IOCAP_ATOMIC8K},
#endif
#ifdef SQLITE_IOCAP_IMMUTABLE
{"SQLITE_IOCAP_IMMUTABLE", SQLITE_IOCAP_IMMUTABLE},
#endif
#ifdef SQLITE_IOCAP_POWERSAFE_OVERWRITE
{"SQLITE_IOCAP_POWERSAFE_OVERWRITE", SQLITE_IOCAP_POWERSAFE_OVERWRITE},
#endif
#ifdef SQLITE_IOCAP_SAFE_APPEND
{"SQLITE_IOCAP_SAFE_APPEND", SQLITE_IOCAP_SAFE_APPEND},
#endif
#ifdef SQLITE_IOCAP_SEQUENTIAL
{"SQLITE_IOCAP_SEQUENTIAL", SQLITE_IOCAP_SEQUENTIAL},
#endif
#ifdef SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN
{"SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN", SQLITE_IOCAP_UNDELETABLE_WHEN_OPEN},
#endif
#ifdef SQLITE_LOCK_EXCLUSIVE
{"SQLITE_LOCK_EXCLUSIVE", SQLITE_LOCK_EXCLUSIVE},
#endif
#ifdef SQLITE_LOCK_NONE
{"SQLITE_LOCK_NONE", SQLITE_LOCK_NONE},
#endif
#ifdef SQLITE_LOCK_PENDING
{"SQLITE_LOCK_PENDING", SQLITE_LOCK_PENDING},
#endif
#ifdef SQLITE_LOCK_RESERVED
{"SQLITE_LOCK_RESERVED", SQLITE_LOCK_RESERVED},
#endif
#ifdef SQLITE_LOCK_SHARED
{"SQLITE_LOCK_SHARED", SQLITE_LOCK_SHARED},
#endif
#ifdef SQLITE_SYNC_DATAONLY
{"SQLITE_SYNC_DATAONLY", SQLITE_SYNC_DATAONLY},
#endif
#ifdef SQLITE_SYNC_FULL
{"SQLITE_SYNC_FULL", SQLITE_SYNC_FULL},
#endif
#ifdef SQLITE_SYNC_NORMAL
{"SQLITE_SYNC_NORMAL", SQLITE_SYNC_NORMAL},
#endif
#ifdef SQLITE_FCNTL_BUSYHANDLER
{"SQLITE_FCNTL_BUSYHANDLER", SQLITE_FCNTL_BUSYHANDLER},
#endif
#ifdef SQLITE_FCNTL_CHUNK_SIZE
{"SQLITE_FCNTL_CHUNK_SIZE", SQLITE_FCNTL_CHUNK_SIZE},
#endif
#ifdef SQLITE_FCNTL_COMMIT_PHASETWO
{"SQLITE_FCNTL_COMMIT_PHASETWO", SQLITE_FCNTL_COMMIT_PHASETWO},
#endif
#ifdef SQLITE_FCNTL_FILE_POINTER
{"SQLITE_FCNTL_FILE_POINTER", SQLITE_FCNTL_FILE_POINTER},
#endif
#ifdef SQLITE_FCNTL_GET_LOCKPROXYFILE
{"SQLITE_FCNTL_GET_LOCKPROXYFILE", SQLITE_FCNTL_GET_LOCKPROXYFILE},
#endif
#ifdef SQLITE_FCNTL_HAS_MOVED
{"SQLITE_FCNTL_HAS_MOVED", SQLITE_FCNTL_HAS_MOVED},
#endif
#ifdef SQLITE_FCNTL_JOURNAL_POINTER
{"SQLITE_FCNTL_JOURNAL_POINTER", SQLITE_FCNTL_JOURNAL_POINTER},
#endif
#ifdef SQLITE_FCNTL_LAST_ERRNO
{"SQLITE_FCNTL_LAST_ERRNO", SQLITE_FCNTL_LAST_ERRNO},
#endif
#ifdef SQLITE_FCNTL_LOCKSTATE
{"SQLITE_FCNTL_LOCKSTATE", SQLITE_FCNTL_LOCKSTATE},
#endif
#ifdef SQLITE_FCNTL_MMAP_SIZE
{"SQLITE_FCNTL_MMAP_SIZE", SQLITE_FCNTL_MMAP_SIZE},
#endif
#ifdef SQLITE_FCNTL_OVERWRITE
{"SQLITE_FCNTL_OVERWRITE", SQLITE_FCNTL_OVERWRITE},
#endif
#ifdef SQLITE_FCNTL_PERSIST_WAL
{"SQLITE_FCNTL_PERSIST_WAL", SQLITE_FCNTL_PERSIST_WAL},
#endif
#ifdef SQLITE_FCNTL_POWERSAFE_OVERWRITE
{"SQLITE_FCNTL_POWERSAFE_OVERWRITE", SQLITE_FCNTL_POWERSAFE_OVERWRITE},
#endif
#ifdef SQLITE_FCNTL_PRAGMA
{"SQLITE_FCNTL_PRAGMA", SQLITE_FCNTL_PRAGMA},
#endif
#ifdef SQLITE_FCNTL_RBU
{"SQLITE_FCNTL_RBU", SQLITE_FCNTL_RBU},
#endif
#ifdef SQLITE_FCNTL_SET_LOCKPROXYFILE
{"SQLITE_FCNTL_SET_LOCKPROXYFILE", SQLITE_FCNTL_SET_LOCKPROXYFILE},
#endif
#ifdef SQLITE_FCNTL_SIZE_HINT
{"SQLITE_FCNTL_SIZE_HINT", SQLITE_FCNTL_SIZE_HINT},
#endif
#ifdef SQLITE_FCNTL_SYNC
{"SQLITE_FCNTL_SYNC", SQLITE_FCNTL_SYNC},
#endif
#ifdef SQLITE_FCNTL_SYNC_OMITTED
{"SQLITE_FCNTL_SYNC_OMITTED", SQLITE_FCNTL_SYNC_OMITTED},
#endif
#ifdef SQLITE_FCNTL_TEMPFILENAME
{"SQLITE_FCNTL_TEMPFILENAME", SQLITE_FCNTL_TEMPFILENAME},
#endif
#ifdef SQLITE_FCNTL_TRACE
{"SQLITE_FCNTL_TRACE", SQLITE_FCNTL_TRACE},
#endif
#ifdef SQLITE_FCNTL_VFSNAME
{"SQLITE_FCNTL_VFSNAME", SQLITE_FCNTL_VFSNAME},
#endif
#ifdef SQLITE_FCNTL_VFS_POINTER
{"SQLITE_FCNTL_VFS_POINTER", SQLITE_FCNTL_VFS_POINTER},
#endif
#ifdef SQLITE_FCNTL_WAL_BLOCK
{"SQLITE_FCNTL_WAL_BLOCK", SQLITE_FCNTL_WAL_BLOCK},
#endif
#ifdef SQLITE_FCNTL_WIN32_AV_RETRY
{"SQLITE_FCNTL_WIN32_AV_RETRY", SQLITE_FCNTL_WIN32_AV_RETRY},
#endif
#ifdef SQLITE_FCNTL_WIN32_SET_HANDLE
{"SQLITE_FCNTL_WIN32_SET_HANDLE", SQLITE_FCNTL_WIN32_SET_HANDLE},
#endif
#ifdef SQLITE_FCNTL_ZIPVFS
{"SQLITE_FCNTL_ZIPVFS", SQLITE_FCNTL_ZIPVFS},
#endif
#ifdef SQLITE_ACCESS_EXISTS
{"SQLITE_ACCESS_EXISTS", SQLITE_ACCESS_EXISTS},
#endif
#ifdef SQLITE_ACCESS_READ
{"SQLITE_ACCESS_READ", SQLITE_ACCESS_READ},
#endif
#ifdef SQLITE_ACCESS_READWRITE
{"SQLITE_ACCESS_READWRITE", SQLITE_ACCESS_READWRITE},
#endif
#ifdef SQLITE_SHM_EXCLUSIVE
{"SQLITE_SHM_EXCLUSIVE", SQLITE_SHM_EXCLUSIVE},
#endif
#ifdef SQLITE_SHM_LOCK
{"SQLITE_SHM_LOCK", SQLITE_SHM_LOCK},
#endif
#ifdef SQLITE_SHM_SHARED
{"SQLITE_SHM_SHARED", SQLITE_SHM_SHARED},
#endif
#ifdef SQLITE_SHM_UNLOCK
{"SQLITE_SHM_UNLOCK", SQLITE_SHM_UNLOCK},
#endif
#ifdef SQLITE_SHM_NLOCK
{"SQLITE_SHM_NLOCK", SQLITE_SHM_NLOCK},
#endif
#ifdef SQLITE_CONFIG_COVERING_INDEX_SCAN
{"SQLITE_CONFIG_COVERING_INDEX_SCAN", SQLITE_CONFIG_COVERING_INDEX_SCAN},
#endif
#ifdef SQLITE_CONFIG_GETMALLOC
{"SQLITE_CONFIG_GETMALLOC", SQLITE_CONFIG_GETMALLOC},
#endif
#ifdef SQLITE_CONFIG_GETMUTEX
{"SQLITE_CONFIG_GETMUTEX", SQLITE_CONFIG_GETMUTEX},
#endif
#ifdef SQLITE_CONFIG_GETPCACHE
{"SQLITE_CONFIG_GETPCACHE", SQLITE_CONFIG_GETPCACHE},
#endif
#ifdef SQLITE_CONFIG_GETPCACHE2
{"SQLITE_CONFIG_GETPCACHE2", SQLITE_CONFIG_GETPCACHE2},
#endif
#ifdef SQLITE_CONFIG_HEAP
{"SQLITE_CONFIG_HEAP", SQLITE_CONFIG_HEAP},
#endif
#ifdef SQLITE_CONFIG_LOG
{"SQLITE_CONFIG_LOG", SQLITE_CONFIG_LOG},
#endif
#ifdef SQLITE_CONFIG_LOOKASIDE
{"SQLITE_CONFIG_LOOKASIDE", SQLITE_CONFIG_LOOKASIDE},
#endif
#ifdef SQLITE_CONFIG_MALLOC
{"SQLITE_CONFIG_MALLOC", SQLITE_CONFIG_MALLOC},
#endif
#ifdef SQLITE_CONFIG_MEMSTATUS
{"SQLITE_CONFIG_MEMSTATUS", SQLITE_CONFIG_MEMSTATUS},
#endif
#ifdef SQLITE_CONFIG_MMAP_SIZE
{"SQLITE_CONFIG_MMAP_SIZE", SQLITE_CONFIG_MMAP_SIZE},
#endif
#ifdef SQLITE_CONFIG_MULTITHREAD
{"SQLITE_CONFIG_MULTITHREAD", SQLITE_CONFIG_MULTITHREAD},
#endif
#ifdef SQLITE_CONFIG_MUTEX
{"SQLITE_CONFIG_MUTEX", SQLITE_CONFIG_MUTEX},
#endif
#ifdef SQLITE_CONFIG_PAGECACHE
{"SQLITE_CONFIG_PAGECACHE", SQLITE_CONFIG_PAGECACHE},
#endif
#ifdef SQLITE_CONFIG_PCACHE
{"SQLITE_CONFIG_PCACHE", SQLITE_CONFIG_PCACHE},
#endif
#ifdef SQLITE_CONFIG_PCACHE2
{"SQLITE_CONFIG_PCACHE2", SQLITE_CONFIG_PCACHE2},
#endif
#ifdef SQLITE_CONFIG_PCACHE_HDRSZ
{"SQLITE_CONFIG_PCACHE_HDRSZ", SQLITE_CONFIG_PCACHE_HDRSZ},
#endif
#ifdef SQLITE_CONFIG_PMASZ
{"SQLITE_CONFIG_PMASZ", SQLITE_CONFIG_PMASZ},
#endif
#ifdef SQLITE_CONFIG_SCRATCH
{"SQLITE_CONFIG_SCRATCH", SQLITE_CONFIG_SCRATCH},
#endif
#ifdef SQLITE_CONFIG_SERIALIZED
{"SQLITE_CONFIG_SERIALIZED", SQLITE_CONFIG_SERIALIZED},
#endif
#ifdef SQLITE_CONFIG_SINGLETHREAD
{"SQLITE_CONFIG_SINGLETHREAD", SQLITE_CONFIG_SINGLETHREAD},
#endif
#ifdef SQLITE_CONFIG_SQLLOG
{"SQLITE_CONFIG_SQLLOG", SQLITE_CONFIG_SQLLOG},
#endif
#ifdef SQLITE_CONFIG_STMTJRNL_SPILL
{"SQLITE_CONFIG_STMTJRNL_SPILL", SQLITE_CONFIG_STMTJRNL_SPILL},
#endif
#ifdef SQLITE_CONFIG_URI
{"SQLITE_CONFIG_URI", SQLITE_CONFIG_URI},
#endif
#ifdef SQLITE_CONFIG_WIN32_HEAPSIZE
{"SQLITE_CONFIG_WIN32_HEAPSIZE", SQLITE_CONFIG_WIN32_HEAPSIZE},
#endif
#ifdef SQLITE_DBCONFIG_ENABLE_FKEY
{"SQLITE_DBCONFIG_ENABLE_FKEY", SQLITE_DBCONFIG_ENABLE_FKEY},
#endif
#ifdef SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER
{"SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER", SQLITE_DBCONFIG_ENABLE_FTS3_TOKENIZER},
#endif
#ifdef SQLITE_DBCONFIG_ENABLE_LOAD_EXTENSION
{"SQLITE_DBCONFIG_ENABLE_LOAD_EXTENSION", SQLITE_DBCONFIG_ENABLE_LOAD_EXTENSION},
#endif
#ifdef SQLITE_DBCONFIG_ENABLE_TRIGGER
{"SQLITE_DBCONFIG_ENABLE_TRIGGER", SQLITE_DBCONFIG_ENABLE_TRIGGER},
#endif
#ifdef SQLITE_DBCONFIG_LOOKASIDE
{"SQLITE_DBCONFIG_LOOKASIDE", SQLITE_DBCONFIG_LOOKASIDE},
#endif
#ifdef SQLITE_DENY
{"SQLITE_DENY", SQLITE_DENY},
#endif
#ifdef SQLITE_IGNORE
{"SQLITE_IGNORE", SQLITE_IGNORE},
#endif
#ifdef SQLITE_ALTER_TABLE
{"SQLITE_ALTER_TABLE", SQLITE_ALTER_TABLE},
#endif
#ifdef SQLITE_ANALYZE
{"SQLITE_ANALYZE", SQLITE_ANALYZE},
#endif
#ifdef SQLITE_ATTACH
{"SQLITE_ATTACH", SQLITE_ATTACH},
#endif
#ifdef SQLITE_COPY
{"SQLITE_COPY", SQLITE_COPY},
#endif
#ifdef SQLITE_CREATE_INDEX
{"SQLITE_CREATE_INDEX", SQLITE_CREATE_INDEX},
#endif
#ifdef SQLITE_CREATE_TABLE
{"SQLITE_CREATE_TABLE", SQLITE_CREATE_TABLE},
#endif
#ifdef SQLITE_CREATE_TEMP_INDEX
{"SQLITE_CREATE_TEMP_INDEX", SQLITE_CREATE_TEMP_INDEX},
#endif
#ifdef SQLITE_CREATE_TEMP_TABLE
{"SQLITE_CREATE_TEMP_TABLE", SQLITE_CREATE_TEMP_TABLE},
#endif
#ifdef SQLITE_CREATE_TEMP_TRIGGER
{"SQLITE_CREATE_TEMP_TRIGGER", SQLITE_CREATE_TEMP_TRIGGER},
#endif
#ifdef SQLITE_CREATE_TEMP_VIEW
{"SQLITE_CREATE_TEMP_VIEW", SQLITE_CREATE_TEMP_VIEW},
#endif
#ifdef SQLITE_CREATE_TRIGGER
{"SQLITE_CREATE_TRIGGER", SQLITE_CREATE_TRIGGER},
#endif
#ifdef SQLITE_CREATE_VIEW
{"SQLITE_CREATE_VIEW", SQLITE_CREATE_VIEW},
#endif
#ifdef SQLITE_CREATE_VTABLE
{"SQLITE_CREATE_VTABLE", SQLITE_CREATE_VTABLE},
#endif
#ifdef SQLITE_DELETE
{"SQLITE_DELETE", SQLITE_DELETE},
#endif
#ifdef SQLITE_DETACH
{"SQLITE_DETACH", SQLITE_DETACH},
#endif
#ifdef SQLITE_DROP_INDEX
{"SQLITE_DROP_INDEX", SQLITE_DROP_INDEX},
#endif
#ifdef SQLITE_DROP_TABLE
{"SQLITE_DROP_TABLE", SQLITE_DROP_TABLE},
#endif
#ifdef SQLITE_DROP_TEMP_INDEX
{"SQLITE_DROP_TEMP_INDEX", SQLITE_DROP_TEMP_INDEX},
#endif
#ifdef SQLITE_DROP_TEMP_TABLE
{"SQLITE_DROP_TEMP_TABLE", SQLITE_DROP_TEMP_TABLE},
#endif
#ifdef SQLITE_DROP_TEMP_TRIGGER
{"SQLITE_DROP_TEMP_TRIGGER", SQLITE_DROP_TEMP_TRIGGER},
#endif
#ifdef SQLITE_DROP_TEMP_VIEW
{"SQLITE_DROP_TEMP_VIEW", SQLITE_DROP_TEMP_VIEW},
#endif
#ifdef SQLITE_DROP_TRIGGER
{"SQLITE_DROP_TRIGGER", SQLITE_DROP_TRIGGER},
#endif
#ifdef SQLITE_DROP_VIEW
{"SQLITE_DROP_VIEW", SQLITE_DROP_VIEW},
#endif
#ifdef SQLITE_DROP_VTABLE
{"SQLITE_DROP_VTABLE", SQLITE_DROP_VTABLE},
#endif
#ifdef SQLITE_FUNCTION
{"SQLITE_FUNCTION", SQLITE_FUNCTION},
#endif
#ifdef SQLITE_INSERT
{"SQLITE_INSERT", SQLITE_INSERT},
#endif
#ifdef SQLITE_PRAGMA
{"SQLITE_PRAGMA", SQLITE_PRAGMA},
#endif
#ifdef SQLITE_READ
{"SQLITE_READ", SQLITE_READ},
#endif
#ifdef SQLITE_RECURSIVE
{"SQLITE_RECURSIVE", SQLITE_RECURSIVE},
#endif
#ifdef SQLITE_REINDEX
{"SQLITE_REINDEX", SQLITE_REINDEX},
#endif
#ifdef SQLITE_SAVEPOINT
{"SQLITE_SAVEPOINT", SQLITE_SAVEPOINT},
#endif
#ifdef SQLITE_SELECT
{"SQLITE_SELECT", SQLITE_SELECT},
#endif
#ifdef SQLITE_TRANSACTION
{"SQLITE_TRANSACTION", SQLITE_TRANSACTION},
#endif
#ifdef SQLITE_UPDATE
{"SQLITE_UPDATE", SQLITE_UPDATE},
#endif
#ifdef SQLITE_LIMIT_ATTACHED
{"SQLITE_LIMIT_ATTACHED", SQLITE_LIMIT_ATTACHED},
#endif
#ifdef SQLITE_LIMIT_COLUMN
{"SQLITE_LIMIT_COLUMN", SQLITE_LIMIT_COLUMN},
#endif
#ifdef SQLITE_LIMIT_COMPOUND_SELECT
{"SQLITE_LIMIT_COMPOUND_SELECT", SQLITE_LIMIT_COMPOUND_SELECT},
#endif
#ifdef SQLITE_LIMIT_EXPR_DEPTH
{"SQLITE_LIMIT_EXPR_DEPTH", SQLITE_LIMIT_EXPR_DEPTH},
#endif
#ifdef SQLITE_LIMIT_FUNCTION_ARG
{"SQLITE_LIMIT_FUNCTION_ARG", SQLITE_LIMIT_FUNCTION_ARG},
#endif
#ifdef SQLITE_LIMIT_LENGTH
{"SQLITE_LIMIT_LENGTH", SQLITE_LIMIT_LENGTH},
#endif
#ifdef SQLITE_LIMIT_LIKE_PATTERN_LENGTH
{"SQLITE_LIMIT_LIKE_PATTERN_LENGTH", SQLITE_LIMIT_LIKE_PATTERN_LENGTH},
#endif
#ifdef SQLITE_LIMIT_SQL_LENGTH
{"SQLITE_LIMIT_SQL_LENGTH", SQLITE_LIMIT_SQL_LENGTH},
#endif
#ifdef SQLITE_LIMIT_TRIGGER_DEPTH
{"SQLITE_LIMIT_TRIGGER_DEPTH", SQLITE_LIMIT_TRIGGER_DEPTH},
#endif
#ifdef SQLITE_LIMIT_VARIABLE_NUMBER
{"SQLITE_LIMIT_VARIABLE_NUMBER", SQLITE_LIMIT_VARIABLE_NUMBER},
#endif
#ifdef SQLITE_LIMIT_VDBE_OP
{"SQLITE_LIMIT_VDBE_OP", SQLITE_LIMIT_VDBE_OP},
#endif
#ifdef SQLITE_LIMIT_WORKER_THREADS
{"SQLITE_LIMIT_WORKER_THREADS", SQLITE_LIMIT_WORKER_THREADS},
#endif
#ifdef SQLITE_BLOB
{"SQLITE_BLOB", SQLITE_BLOB},
#endif
#ifdef SQLITE_FLOAT
{"SQLITE_FLOAT", SQLITE_FLOAT},
#endif
#ifdef SQLITE_INTEGER
{"SQLITE_INTEGER", SQLITE_INTEGER},
#endif
#ifdef SQLITE_NULL
{"SQLITE_NULL", SQLITE_NULL},
#endif
#ifdef SQLITE_TEXT
{"SQLITE_TEXT", SQLITE_TEXT},
#endif
#ifdef SQLITE_ANY
{"SQLITE_ANY", SQLITE_ANY},
#endif
#ifdef SQLITE_UTF16
{"SQLITE_UTF16", SQLITE_UTF16},
#endif
#ifdef SQLITE_UTF16BE
{"SQLITE_UTF16BE", SQLITE_UTF16BE},
#endif
#ifdef SQLITE_UTF16LE
{"SQLITE_UTF16LE", SQLITE_UTF16LE},
#endif
#ifdef SQLITE_UTF16_ALIGNED
{"SQLITE_UTF16_ALIGNED", SQLITE_UTF16_ALIGNED},
#endif
#ifdef SQLITE_UTF8
{"SQLITE_UTF8", SQLITE_UTF8},
#endif
#ifdef SQLITE_DETERMINISTIC
{"SQLITE_DETERMINISTIC", SQLITE_DETERMINISTIC},
#endif
#ifdef SQLITE_STATIC
{"SQLITE_STATIC", SQLITE_STATIC},
#endif
#ifdef SQLITE_TRANSIENT
{"SQLITE_TRANSIENT", SQLITE_TRANSIENT},
#endif
#ifdef SQLITE_INDEX_SCAN_UNIQUE
{"SQLITE_INDEX_SCAN_UNIQUE", SQLITE_INDEX_SCAN_UNIQUE},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_EQ
{"SQLITE_INDEX_CONSTRAINT_EQ", SQLITE_INDEX_CONSTRAINT_EQ},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_GE
{"SQLITE_INDEX_CONSTRAINT_GE", SQLITE_INDEX_CONSTRAINT_GE},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_GLOB
{"SQLITE_INDEX_CONSTRAINT_GLOB", SQLITE_INDEX_CONSTRAINT_GLOB},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_GT
{"SQLITE_INDEX_CONSTRAINT_GT", SQLITE_INDEX_CONSTRAINT_GT},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_LE
{"SQLITE_INDEX_CONSTRAINT_LE", SQLITE_INDEX_CONSTRAINT_LE},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_LIKE
{"SQLITE_INDEX_CONSTRAINT_LIKE", SQLITE_INDEX_CONSTRAINT_LIKE},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_LT
{"SQLITE_INDEX_CONSTRAINT_LT", SQLITE_INDEX_CONSTRAINT_LT},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_MATCH
{"SQLITE_INDEX_CONSTRAINT_MATCH", SQLITE_INDEX_CONSTRAINT_MATCH},
#endif
#ifdef SQLITE_INDEX_CONSTRAINT_REGEXP
{"SQLITE_INDEX_CONSTRAINT_REGEXP", SQLITE_INDEX_CONSTRAINT_REGEXP},
#endif
#ifdef SQLITE_MUTEX_FAST
{"SQLITE_MUTEX_FAST", SQLITE_MUTEX_FAST},
#endif
#ifdef SQLITE_MUTEX_RECURSIVE
{"SQLITE_MUTEX_RECURSIVE", SQLITE_MUTEX_RECURSIVE},
#endif
#ifdef SQLITE_MUTEX_STATIC_APP1
{"SQLITE_MUTEX_STATIC_APP1", SQLITE_MUTEX_STATIC_APP1},
#endif
#ifdef SQLITE_MUTEX_STATIC_APP2
{"SQLITE_MUTEX_STATIC_APP2", SQLITE_MUTEX_STATIC_APP2},
#endif
#ifdef SQLITE_MUTEX_STATIC_APP3
{"SQLITE_MUTEX_STATIC_APP3", SQLITE_MUTEX_STATIC_APP3},
#endif
#ifdef SQLITE_MUTEX_STATIC_LRU
{"SQLITE_MUTEX_STATIC_LRU", SQLITE_MUTEX_STATIC_LRU},
#endif
#ifdef SQLITE_MUTEX_STATIC_LRU2
{"SQLITE_MUTEX_STATIC_LRU2", SQLITE_MUTEX_STATIC_LRU2},
#endif
#ifdef SQLITE_MUTEX_STATIC_MASTER
{"SQLITE_MUTEX_STATIC_MASTER", SQLITE_MUTEX_STATIC_MASTER},
#endif
#ifdef SQLITE_MUTEX_STATIC_MEM
{"SQLITE_MUTEX_STATIC_MEM", SQLITE_MUTEX_STATIC_MEM},
#endif
#ifdef SQLITE_MUTEX_STATIC_MEM2
{"SQLITE_MUTEX_STATIC_MEM2", SQLITE_MUTEX_STATIC_MEM2},
#endif
#ifdef SQLITE_MUTEX_STATIC_OPEN
{"SQLITE_MUTEX_STATIC_OPEN", SQLITE_MUTEX_STATIC_OPEN},
#endif
#ifdef SQLITE_MUTEX_STATIC_PMEM
{"SQLITE_MUTEX_STATIC_PMEM", SQLITE_MUTEX_STATIC_PMEM},
#endif
#ifdef SQLITE_MUTEX_STATIC_PRNG
{"SQLITE_MUTEX_STATIC_PRNG", SQLITE_MUTEX_STATIC_PRNG},
#endif
#ifdef SQLITE_MUTEX_STATIC_VFS1
{"SQLITE_MUTEX_STATIC_VFS1", SQLITE_MUTEX_STATIC_VFS1},
#endif
#ifdef SQLITE_MUTEX_STATIC_VFS2
{"SQLITE_MUTEX_STATIC_VFS2", SQLITE_MUTEX_STATIC_VFS2},
#endif
#ifdef SQLITE_MUTEX_STATIC_VFS3
{"SQLITE_MUTEX_STATIC_VFS3", SQLITE_MUTEX_STATIC_VFS3},
#endif
#ifdef SQLITE_TESTCTRL_ALWAYS
{"SQLITE_TESTCTRL_ALWAYS", SQLITE_TESTCTRL_ALWAYS},
#endif
#ifdef SQLITE_TESTCTRL_ASSERT
{"SQLITE_TESTCTRL_ASSERT", SQLITE_TESTCTRL_ASSERT},
#endif
#ifdef SQLITE_TESTCTRL_BENIGN_MALLOC_HOOKS
{"SQLITE_TESTCTRL_BENIGN_MALLOC_HOOKS", SQLITE_TESTCTRL_BENIGN_MALLOC_HOOKS},
#endif
#ifdef SQLITE_TESTCTRL_BITVEC_TEST
{"SQLITE_TESTCTRL_BITVEC_TEST", SQLITE_TESTCTRL_BITVEC_TEST},
#endif
#ifdef SQLITE_TESTCTRL_BYTEORDER
{"SQLITE_TESTCTRL_BYTEORDER", SQLITE_TESTCTRL_BYTEORDER},
#endif
#ifdef SQLITE_TESTCTRL_EXPLAIN_STMT
{"SQLITE_TESTCTRL_EXPLAIN_STMT", SQLITE_TESTCTRL_EXPLAIN_STMT},
#endif
#ifdef SQLITE_TESTCTRL_FAULT_INSTALL
{"SQLITE_TESTCTRL_FAULT_INSTALL", SQLITE_TESTCTRL_FAULT_INSTALL},
#endif
#ifdef SQLITE_TESTCTRL_FIRST
{"SQLITE_TESTCTRL_FIRST", SQLITE_TESTCTRL_FIRST},
#endif
#ifdef SQLITE_TESTCTRL_IMPOSTER
{"SQLITE_TESTCTRL_IMPOSTER", SQLITE_TESTCTRL_IMPOSTER},
#endif
#ifdef SQLITE_TESTCTRL_ISINIT
{"SQLITE_TESTCTRL_ISINIT", SQLITE_TESTCTRL_ISINIT},
#endif
#ifdef SQLITE_TESTCTRL_ISKEYWORD
{"SQLITE_TESTCTRL_ISKEYWORD", SQLITE_TESTCTRL_ISKEYWORD},
#endif
#ifdef SQLITE_TESTCTRL_LAST
{"SQLITE_TESTCTRL_LAST", SQLITE_TESTCTRL_LAST},
#endif
#ifdef SQLITE_TESTCTRL_LOCALTIME_FAULT
{"SQLITE_TESTCTRL_LOCALTIME_FAULT", SQLITE_TESTCTRL_LOCALTIME_FAULT},
#endif
#ifdef SQLITE_TESTCTRL_NEVER_CORRUPT
{"SQLITE_TESTCTRL_NEVER_CORRUPT", SQLITE_TESTCTRL_NEVER_CORRUPT},
#endif
#ifdef SQLITE_TESTCTRL_OPTIMIZATIONS
{"SQLITE_TESTCTRL_OPTIMIZATIONS", SQLITE_TESTCTRL_OPTIMIZATIONS},
#endif
#ifdef SQLITE_TESTCTRL_PENDING_BYTE
{"SQLITE_TESTCTRL_PENDING_BYTE", SQLITE_TESTCTRL_PENDING_BYTE},
#endif
#ifdef SQLITE_TESTCTRL_PRNG_RESET
{"SQLITE_TESTCTRL_PRNG_RESET", SQLITE_TESTCTRL_PRNG_RESET},
#endif
#ifdef SQLITE_TESTCTRL_PRNG_RESTORE
{"SQLITE_TESTCTRL_PRNG_RESTORE", SQLITE_TESTCTRL_PRNG_RESTORE},
#endif
#ifdef SQLITE_TESTCTRL_PRNG_SAVE
{"SQLITE_TESTCTRL_PRNG_SAVE", SQLITE_TESTCTRL_PRNG_SAVE},
#endif
#ifdef SQLITE_TESTCTRL_RESERVE
{"SQLITE_TESTCTRL_RESERVE", SQLITE_TESTCTRL_RESERVE},
#endif
#ifdef SQLITE_TESTCTRL_SCRATCHMALLOC
{"SQLITE_TESTCTRL_SCRATCHMALLOC", SQLITE_TESTCTRL_SCRATCHMALLOC},
#endif
#ifdef SQLITE_TESTCTRL_SORTER_MMAP
{"SQLITE_TESTCTRL_SORTER_MMAP", SQLITE_TESTCTRL_SORTER_MMAP},
#endif
#ifdef SQLITE_TESTCTRL_VDBE_COVERAGE
{"SQLITE_TESTCTRL_VDBE_COVERAGE", SQLITE_TESTCTRL_VDBE_COVERAGE},
#endif
#ifdef SQLITE_STATUS_MALLOC_COUNT
{"SQLITE_STATUS_MALLOC_COUNT", SQLITE_STATUS_MALLOC_COUNT},
#endif
#ifdef SQLITE_STATUS_MALLOC_SIZE
{"SQLITE_STATUS_MALLOC_SIZE", SQLITE_STATUS_MALLOC_SIZE},
#endif
#ifdef SQLITE_STATUS_MEMORY_USED
{"SQLITE_STATUS_MEMORY_USED", SQLITE_STATUS_MEMORY_USED},
#endif
#ifdef SQLITE_STATUS_PAGECACHE_OVERFLOW
{"SQLITE_STATUS_PAGECACHE_OVERFLOW", SQLITE_STATUS_PAGECACHE_OVERFLOW},
#endif
#ifdef SQLITE_STATUS_PAGECACHE_SIZE
{"SQLITE_STATUS_PAGECACHE_SIZE", SQLITE_STATUS_PAGECACHE_SIZE},
#endif
#ifdef SQLITE_STATUS_PAGECACHE_USED
{"SQLITE_STATUS_PAGECACHE_USED", SQLITE_STATUS_PAGECACHE_USED},
#endif
#ifdef SQLITE_STATUS_PARSER_STACK
{"SQLITE_STATUS_PARSER_STACK", SQLITE_STATUS_PARSER_STACK},
#endif
#ifdef SQLITE_STATUS_SCRATCH_OVERFLOW
{"SQLITE_STATUS_SCRATCH_OVERFLOW", SQLITE_STATUS_SCRATCH_OVERFLOW},
#endif
#ifdef SQLITE_STATUS_SCRATCH_SIZE
{"SQLITE_STATUS_SCRATCH_SIZE", SQLITE_STATUS_SCRATCH_SIZE},
#endif
#ifdef SQLITE_STATUS_SCRATCH_USED
{"SQLITE_STATUS_SCRATCH_USED", SQLITE_STATUS_SCRATCH_USED},
#endif
#ifdef SQLITE_DBSTATUS_CACHE_HIT
{"SQLITE_DBSTATUS_CACHE_HIT", SQLITE_DBSTATUS_CACHE_HIT},
#endif
#ifdef SQLITE_DBSTATUS_CACHE_MISS
{"SQLITE_DBSTATUS_CACHE_MISS", SQLITE_DBSTATUS_CACHE_MISS},
#endif
#ifdef SQLITE_DBSTATUS_CACHE_USED
{"SQLITE_DBSTATUS_CACHE_USED", SQLITE_DBSTATUS_CACHE_USED},
#endif
#ifdef SQLITE_DBSTATUS_CACHE_WRITE
{"SQLITE_DBSTATUS_CACHE_WRITE", SQLITE_DBSTATUS_CACHE_WRITE},
#endif
#ifdef SQLITE_DBSTATUS_DEFERRED_FKS
{"SQLITE_DBSTATUS_DEFERRED_FKS", SQLITE_DBSTATUS_DEFERRED_FKS},
#endif
#ifdef SQLITE_DBSTATUS_LOOKASIDE_HIT
{"SQLITE_DBSTATUS_LOOKASIDE_HIT", SQLITE_DBSTATUS_LOOKASIDE_HIT},
#endif
#ifdef SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL
{"SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL", SQLITE_DBSTATUS_LOOKASIDE_MISS_FULL},
#endif
#ifdef SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE
{"SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE", SQLITE_DBSTATUS_LOOKASIDE_MISS_SIZE},
#endif
#ifdef SQLITE_DBSTATUS_LOOKASIDE_USED
{"SQLITE_DBSTATUS_LOOKASIDE_USED", SQLITE_DBSTATUS_LOOKASIDE_USED},
#endif
#ifdef SQLITE_DBSTATUS_MAX
{"SQLITE_DBSTATUS_MAX", SQLITE_DBSTATUS_MAX},
#endif
#ifdef SQLITE_DBSTATUS_SCHEMA_USED
{"SQLITE_DBSTATUS_SCHEMA_USED", SQLITE_DBSTATUS_SCHEMA_USED},
#endif
#ifdef SQLITE_DBSTATUS_STMT_USED
{"SQLITE_DBSTATUS_STMT_USED", SQLITE_DBSTATUS_STMT_USED},
#endif
#ifdef SQLITE_STMTSTATUS_AUTOINDEX
{"SQLITE_STMTSTATUS_AUTOINDEX", SQLITE_STMTSTATUS_AUTOINDEX},
#endif
#ifdef SQLITE_STMTSTATUS_FULLSCAN_STEP
{"SQLITE_STMTSTATUS_FULLSCAN_STEP", SQLITE_STMTSTATUS_FULLSCAN_STEP},
#endif
#ifdef SQLITE_STMTSTATUS_SORT
{"SQLITE_STMTSTATUS_SORT", SQLITE_STMTSTATUS_SORT},
#endif
#ifdef SQLITE_STMTSTATUS_VM_STEP
{"SQLITE_STMTSTATUS_VM_STEP", SQLITE_STMTSTATUS_VM_STEP},
#endif
#ifdef SQLITE_CHECKPOINT_FULL
{"SQLITE_CHECKPOINT_FULL", SQLITE_CHECKPOINT_FULL},
#endif
#ifdef SQLITE_CHECKPOINT_PASSIVE
{"SQLITE_CHECKPOINT_PASSIVE", SQLITE_CHECKPOINT_PASSIVE},
#endif
#ifdef SQLITE_CHECKPOINT_RESTART
{"SQLITE_CHECKPOINT_RESTART", SQLITE_CHECKPOINT_RESTART},
#endif
#ifdef SQLITE_CHECKPOINT_TRUNCATE
{"SQLITE_CHECKPOINT_TRUNCATE", SQLITE_CHECKPOINT_TRUNCATE},
#endif
#ifdef SQLITE_VTAB_CONSTRAINT_SUPPORT
{"SQLITE_VTAB_CONSTRAINT_SUPPORT", SQLITE_VTAB_CONSTRAINT_SUPPORT},
#endif
#ifdef SQLITE_FAIL
{"SQLITE_FAIL", SQLITE_FAIL},
#endif
#ifdef SQLITE_REPLACE
{"SQLITE_REPLACE", SQLITE_REPLACE},
#endif
#ifdef SQLITE_ROLLBACK
{"SQLITE_ROLLBACK", SQLITE_ROLLBACK},
#endif
#ifdef SQLITE_SCANSTAT_EST
{"SQLITE_SCANSTAT_EST", SQLITE_SCANSTAT_EST},
#endif
#ifdef SQLITE_SCANSTAT_EXPLAIN
{"SQLITE_SCANSTAT_EXPLAIN", SQLITE_SCANSTAT_EXPLAIN},
#endif
#ifdef SQLITE_SCANSTAT_NAME
{"SQLITE_SCANSTAT_NAME", SQLITE_SCANSTAT_NAME},
#endif
#ifdef SQLITE_SCANSTAT_NLOOP
{"SQLITE_SCANSTAT_NLOOP", SQLITE_SCANSTAT_NLOOP},
#endif
#ifdef SQLITE_SCANSTAT_NVISIT
{"SQLITE_SCANSTAT_NVISIT", SQLITE_SCANSTAT_NVISIT},
#endif
#ifdef SQLITE_SCANSTAT_SELECTID
{"SQLITE_SCANSTAT_SELECTID", SQLITE_SCANSTAT_SELECTID},
#endif
